-- Most of this file is largely borrowed from Pitbull_Luatexts, thanks to ckknight
local MAJOR = "LibScriptableDisplayPluginColor-1.0"
local MINOR = 15

local ScriptEnv = {}

local PluginColor = LibStub:NewLibrary(MAJOR, MINOR)
if not PluginColor then return end

-- Populate an environment with this plugin's fields
-- @usage :New(environment) 
-- @parma environment This will be the environment when setfenv is called.
-- @return A new plugin object, aka the environment
function PluginColor:New(environment)
	for k, v in pairs(ScriptEnv) do
		environment[k] = v
	end
	
	environment.gradient = self:RedThroughBlue()
	
	return environment
end

local LibEvaluator = LibStub("LibScriptableDisplayEvaluator-1.0")
local pool = setmetatable({}, {__mode = "k"})

local function new()
	local obj = next(pool)
	
	if obj then
		pool[obj] = nil
	else
		obj = {}
	end
	
	return obj
end

local function del(tbl)
	pool[tbl] = true
end

local function Color2RGBA(color, thirtyTwoBit, environment)
	
	local r, g, b, a = 0, 0, 0, 0
		
	if (color == nil) then
		return -1;
	end

	local l = color
		
	if thirtyTwoBit then
		--/* RGBA */
		a = bit.band(bit.rshift(l, 24), 0xff) / 255
		r = bit.band(bit.rshift(l, 16), 0xff) / 255
		g = bit.band(bit.rshift(l, 8), 0xff) / 255
		b = bit.band(bit.rshift(l, 0), 0xff) / 255
	else
		--/* RGB */
		r = bit.band(bit.rshift(l, 16), 0xff) / 255
		g = bit.band(bit.rshift(l, 8), 0xff) / 255
		b = bit.band(bit.rshift(l, 0), 0xff) / 255
		a = 0xff000000 / 255
	end
	return r, g, b, a
end
ScriptEnv.Color2RGBA = Color2RGBA

local function RGBA2Color(red, green, blue, alpha)
	if alpha then
		return bit.bor(bit.bor(bit.bor(bit.lshift(red, 16), bit.lshift(green, 8)) + bit.lshift(blue, 8)), bit.lshift(alpha, 24))
	else
		return bit.bor(bit.bor(bit.lshift(red, 16), bit.lshift(green, 8)), blue)
	end
end
ScriptEnv.RGBA2Color = RGBA2Color

local function RGB2Gray(red, green, blue)
	return (77 * red + 150 * green + 28 * blue) / 255;
end
ScriptEnv.RGB2Gray = RGB2Gray

local function RGB2Black(red, green, blue)
	return RGB2Gray(red, green, blue) < 127
end
ScriptEnv.RGB2Black = RGB2Black

local function BgColor(r, g, b)
	return (r + 0.2)/3, (g + 0.2)/3, (b + 0.2)/3
end
ScriptEnv.BgColor = BgColor

local function ThreatStatusColor(status)
	local r, g, b = GetThreatStatusColor(status)
	return r * 255, g * 255, b * 255
end
ScriptEnv.ThreatStatusColor = ThreatStatusColor

-- This comes from oUF
local function Gradient(perc)
    if perc <= 0.5 then
        return 1, perc*2, 0
    else
        return 2 - perc*2, 1, 0
    end
end
ScriptEnv.Gradient = Gradient

function PluginColor:RedThroughBlue()
	local colors = {}

	-- Start with solid red:
	local r = 255;
	local g = 0;
	local b = 0;
	
	-- From red to yellow:
	local i = 0
	while i <= 255 do
		g = i
		tinsert(colors, {r / 255, g / 255, b / 255})
		i = i + 1
	end

	-- From yellow to green:
	i = 255
	while i >= 0 do
		r = i
		tinsert(colors, {r / 255, g / 255, b / 255})
		i = i - 1
	end
	
	-- From green to blue:
	i = 0
	while i <= 255 do
		b = i
		tinsert(colors, {r / 255, g / 255, b / 255})
		g = g - 1
		i = i + 1
	end

	local subColors = {}
	
	for i = #colors, 1, -1 do
		if mod(i, 8) == 0 then
			tinsert(subColors, colors[i])
		end
	end
	
	for i = #subColors + 1, 100 do
		tinsert(subColors, colors[1])
	end

	return subColors
end
ScriptEnv.RedThroughBlue = RedThroughBlue

local function Colorize(str, r, g, b)
	if type(str) ~= "string" then return "" end
	if type(r) ~= "number" then r = 1 end
	if type(g) ~= "number" then g = 1 end
	if type(b) ~= "number" then b = 1 end
	return ("|cff%02x%02x%02x%s |r"):format(r * 255, g * 255, b * 255, str)
end
ScriptEnv.Colorize = Colorize

local function AggroColor(unit)
	local r, g, b = UnitSelectionColor(unit)
	return r, g, b
end
ScriptEnv.AggroColor = AggroColor

local function ThreatStatusColor(status)
	local r, g, b = GetThreatStatusColor(status)
	return r, g, b
end
ScriptEnv.ThreatStatusColor = ThreatStatusColor

local function HPColor(cur, max)
	local perc = cur / max
	local r1, g1, b1
	local r2, g2, b2
	if perc <= 0.5 then
		perc = perc * 2
		r1, g1, b1 = 1, 0, 0 
		r2, g2, b2 = 1, 1, 0
	else
		perc = perc * 2 - 1
		r1, g1, b1 = 1, 1, 0
		r2, g2, b2 = 0, 1, 0
	end
	local r, g, b = r1 + (r2 - r1)*perc, g1 + (g2 - g1)*perc, b1 + (b2 - b1)*perc
	if r < 0 then
		r = 0
	elseif r > 1 then
		r = 1
	end
	if g < 0 then
		g = 0
	elseif g > 1 then
		g = 1
	end
	if b < 0 then
		b = 0
	elseif b > 1 then
		b = 1
	end
	
	return r, g, b
end
ScriptEnv.HPColor = HPColor

local PowerBarColor = _G.PowerBarColor

local PowerTypes = {Warrior = "RAGE", Rogue = "ENERGY", Pet = "HAPPINESS", Deathknight = "RUNIC_POWER", Hunter = "FOCUS"}

setmetatable(PowerTypes, {__index = function(self, key)
	if rawget(self, key) then return rawget(self, key) end
	return "MANA"
end})

local function PowerColor(power_type, unit)
	if unit then 
		local class = UnitClass(unit)
		power_type = PowerTypes[class]
	end
	local color = PowerBarColor[power_type]
	local r,g,b
	if color then
		r, g, b = color.r,color.g,color.b
	else
		r, g, b = 0.7, 0.7, 0.7
	end
	return r, g, b
end
ScriptEnv.PowerColor = PowerColor

local function ReputationColor(reaction)
  local color = FACTION_BAR_COLORS[reaction]
	if color then
		return color.r, color.g, color.b
	end
end
ScriptEnv.ReputationColor = ReputationColor

local FACTION_BAR_COLORS = _G.FACTION_BAR_COLORS
local HOSTILE_REACTION = 2
local NEUTRAL_REACTION = 4
local FRIENDLY_REACTION = 5

local function HostileColor(unit)
	local r, g, b
	if not unit then
		r, g, b = 0.8, 0.8, 0.8 --UNKNOWN
	else
		if UnitIsPlayer(unit) or UnitPlayerControlled(unit) then
			if UnitCanAttack(unit, "player") then
				-- they can attack me
				if UnitCanAttack("player", unit) then
					-- and I can attack them
					local t = FACTION_BAR_COLORS[HOSTILE_REACTION]
					r, g, b = t.r, t.g, t.b
				else
					-- but I can't attack them
--					r, g, b = unpack(FACTION_BAR_COLORS.civilian)
					r, g, b = 0.8, 0.8, 0.8
				end
			elseif UnitCanAttack("player", unit) then
				-- they can't attack me, but I can attack them
				local t = (FACTION_BAR_COLORS[NEUTRAL_REACTION])
				r, g, b = t.r, t.g, t.b
			elseif UnitIsPVP(unit) then
				-- on my team
				local t = FACTION_BAR_COLORS[FRIENDLY_REACTION]
				r, g, b = t.r, t.g, t.b
			else
				-- either enemy or friend, no violance
				-- r, g, b = unpack(PitBull4.ReactionColors.civilian)
				r, g, b = 0.8, 0.8, 0.8
			end
		elseif (UnitIsTapped(unit) and not UnitIsTappedByPlayer(unit)) or UnitIsDead(unit) then
			r, g, b = 0.5, 0.5, 0.5 -- TODO: We really need this to be globally configurable.
		else
			local reaction = UnitReaction(unit, "player")
			if reaction then
				local t
				if reaction >= 5 then
					t = FACTION_BAR_COLORS[FRIENDLY_REACTION]
				elseif reaction == 4 then
					t = FACTION_BAR_COLORS[NEUTRAL_REACTION]
				else
					t = FACTION_BAR_COLORS[HOSTILE_REACTION]
				end
				r, g, b = t.r, t.g, t.b
			else
				r, g, b = 0.8, 0.8, 0.8 --UNKNOWN
			end
		end
	end
	return r, g, b
end
ScriptEnv.HostileColor = HostileColor

local RAID_CLASS_COLORS = _G.RAID_CLASS_COLORS

local function ClassColor(unit)
	local r, g, b = 0.8, 0.8, 0.8 --UNKNOWN
	local _, class = UnitClass(unit)
	local t = RAID_CLASS_COLORS[class]
	
	if t then
		r, g, b = t.r, t.g, t.b
	end
	
	return r, g, b
end
ScriptEnv.ClassColor = ClassColor

local function DifficultyColor(unit)
	local level
	level = UnitLevel(unit)
	if level <= 0 then
		level = 99
	end
	local color = GetQuestDifficultyColor(level)
	return color.r, color.g, color.b
end
ScriptEnv.DifficultyColor = DifficultyColor

